# -*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE!
# This file has been autogenerated by dephell <3
# https://github.com/dephell/dephell

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

readme = ''

setup(
    long_description=readme,
    name='packratt',
    version='0.1.2',
    python_requires='==3.*,>=3.6.0',
    author='Simon Perkins',
    author_email='simon.perkins@gmail.com',
    entry_points={"console_scripts": ["packratt = packratt.application:run"]},
    packages=['packratt', 'packratt.tests'],
    package_dir={"": "."},
    package_data={"packratt": ["conf/*.yaml"]},
    install_requires=[
        'appdirs==1.*,>=1.4.4', 'jsonschema==3.*,>=3.2.0',
        'pyyaml==5.*,>=5.3.1', 'requests==2.*,>=2.23.0'
    ],
    extras_require={
        "dev": ["pylint==2.*,>=2.5.3"],
        "testing": [
            "pytest[testing]==5.*,>=5.2.0,>=5.4.2",
            "pytest-flake8[testing]==1.*,>=1.0.6"
        ]
    },
)
